var restler = require('restler');

var settings = {};

module.exports = {
	attach: function (expressApp, clientId, redirectdomain, callbackPageHandler) {
		settings.clientId = clientId;
		settings.redirectDomain = redirectdomain;
		callbackPageHandler = callbackPageHandler || function (req, res) {
			res.writeHead(200, {'Content-Type': 'text/html'});
			res.write('Welcome! <a href="/huddle/login">Login</a>');
			res.end();
		};

		expressApp.get('/receiveAuthCode', function (req, res) {
			callbackPageHandler(req, res);
		});

		expressApp.get('/huddleAuth', function (req, res) {
			return res.redirect('https://login.huddle.net/request?response_type=code&client_id=' + clientId + '&redirect_uri=' + encodeURIComponent('https://' + settings.redirectDomain) + '/receiveAuthCode');
		});

		expressApp.post('/huddleAuth/*', function (req, res) {
			var url = 'https://login.huddle.net/' + req.url.substring(12, req.url.length);

			var authData;

			if (req.url.substring(5, req.url.length) === 'refresh') {
				authData = 'grant_type=refresh_token&client_id=' + clientId + '&refresh_token=' + req.body.refresh_token;
			} else {
				authData = 'grant_type=authorization_code&client_id=' + settings.clientId + '&redirect_uri=' + encodeURIComponent('https://' + settings.redirectDomain) + '/receiveAuthCode&code=' + req.body.code;
			}

			console.log('URL is ' + url);

			restler.post(url , {
				headers: {
					'Content-Type': 'application/x-www-form-urlencoded'
				},
				data: authData,
				parser: restler.parsers.json
			}).on('complete', function (data, response) {
				console.log('Data is back: ' + JSON.stringify(data));
				console.log('Access token is ' + data['access_token'] + ' and it expires in ' + data['expires_in'] + ' s and the refresh token is ' + data['refresh_token']);
				res.send(data);
			});
		});

		expressApp.all('/huddle/*', function (req, res) {
			// CHANGE THIS TO WHATEVER YOU'RE POINTING AT
			var apiUrl = 'http://api.huddle.local/' + req.url.substring(8, req.url.length);
			var authData = req.body;
			var method = req.method.toLowerCase();

			console.log('Received a %s request for the API with body %s', req.method, JSON.stringify(authData));

			console.log('Routing for url ' + req.url +  ' to ' + apiUrl);

			var routedHeaders = req.headers;
			delete routedHeaders.location;
			delete routedHeaders.host;

			if(method == 'post') {
				restler.post(apiUrl, {
					headers: routedHeaders,
					data: authData
				}).on('complete', function(data, response) {
					console.log('POST complete: Data is %s, Status is %s', 'data', response.statusCode);
					res.json(data, response.statusCode);
				});
			} else if (method == 'get') {
				restler.get(apiUrl, {
					headers: routedHeaders,
					data: authData
				}).on('complete', function(data, response) {
					console.log('GET complete: Data is %s, Status is %s', 'data', response.statusCode);
					res.json(data, response.statusCode);
				});
			} else if (method == 'put') {
				restler.put(apiUrl, {
					headers: routedHeaders,
					data: authData
				}).on('complete', function(data, response) {
					console.log('PUT complete: Data is %s, Status is %s', 'data', response.statusCode);
					res.json(data, response.statusCode);
				});
			} else if (method == 'delete') {
				restler.del(apiUrl, {
					headers: routedHeaders,
					data: authData
				}).on('complete', function(data, response) {
					console.log('DELETE complete: Data is %s, Status is %s', 'data', response.statusCode);
					res.json(data, response.statusCode);
				});
			}
		});
	}
};